class LogCoshError:
    """
    Logarithm of the hyperbolic cosine of the prediction error.

    Args:
        y_true: Ground truth values.
        y_pred: The predicted values.

    Returns:
        Logcosh error
    """

    def __call__(self, y_pred, y_true):
        diff = y_pred - y_true
        return torch.mean(torch.log((torch.exp(diff) + torch.exp(-1. * diff)) / 2.))


from .rsquared import RSquared
from .mse import MeanSquaredError
from .mae import MeanAbsoluteError
from .rmse import RootMeanSquaredError
from .precision_recall import Precision, Recall
from .f1 import F1Score
from .meaniou import MeanIoU
from .dice import DSC
from .logcosh import LogCoshError
from .accuracy import Accuracy


import torch

class Accuracy:
    def __call__(self, tensor1, tensor2):
        return torch.mean((t1==t2).type(torch.float64))

# metric = DSCMetric()
# t1 = torch.tensor([1., 2., 3., 4.])
# t2 = torch.tensor([0., 2., 3., 4.])
# print(metric(t1, t2))


import torch

class HingeMetric:
    def __call__(self, tensor1, tensor2):
        """
        Arguments
        ---------
        pred : torch.Tensor
        ground_truth : torch.Tensor [-1 or 1]
        """
        if 0. in torch.unique(tensor2):
            tensor2[tensor2 == 0.] = -1.
        hinge_loss = 1 - torch.mul(tensor1, tensor2)
        hinge_loss[hinge_loss < 0] = 0
        return torch.mean(hinge_loss)


# metric = HingeMetric()
# t1 = torch.tensor([[0.6, 0.4], [0.4, 0.6]])
# t2 = torch.tensor([[0., 1.], [0., 0.]])
# print(metric(t1, t2))


import torch

class RootMeanSquaredError:
    def __call__(self, tensor1, tensor2):
        """
        Returns the root mean squared error (RMSE) of two tensors.

        Arguments
        ---------
        x : torch.Tensor
        y : torch.Tensor
        """
        return torch.sqrt(torch.mean((tensor1 - tensor2)**2))